@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -546,9 +544,9 @@ static void cb_handle_msg_event(void *opdata, OtrlMessageEvent event,
 
 #endif /* HAVE_LIBOTR3 */
 
-/*
- * returns whether a otr_message was received
- * sets *otr_data to NULL, when it was an internal otr message
+/*  otr_receive
+ * Returns whether a otr_message was received.
+ * Sets *otr_data to NULL, when it was an internal otr message.
  */
 int otr_receive(char **otr_data, const char *buddy, int *free_msg)
 {
@@ -598,50 +596,78 @@ int otr_receive(char **otr_data, const char *buddy, int *free_msg)
   return 0;
 }
 
-int otr_send(char **msg, const char *buddy)
+/*  otr_send
+ * Returns a newly allocated string, or NULL if no message should be sent.
+ * Sets *encryption_status to 1 if "the otr context is encrypted or otr tries
+ * to establish a session before the message is sent".
+ * (If the session isn't established, but the user choose "otr required", then
+ * the string just contains the "initiate session" message, not the message of
+ * the user.)
+ */
+char *otr_send(const char *msg, const char *buddy, int *encryption_status)
 {
   gcry_error_t err;
   char *newmessage = NULL;
-  char *htmlmsg;
+  char *rmsg;
   ConnContext *ctx = otr_get_context(buddy);
 
+  if (!encryption_status)
+    return NULL;
+
+  *encryption_status = 0;
+
+  if (!buddy || !msg || !msg[0])
+    return NULL;
+
   if (ctx->msgstate == OTRL_MSGSTATE_PLAINTEXT)
     err = otrl_message_sending(userstate, &ops, NULL, ctx->accountname,
 #ifdef HAVE_LIBOTR3
-                               ctx->protocol, ctx->username, *msg, NULL,
+                               ctx->protocol, ctx->username, msg, NULL,
                                &newmessage, NULL, NULL);
 #else
                                // INSTAG XXX
                                ctx->protocol, ctx->username, OTRL_INSTAG_BEST,
-                               *msg, NULL, &newmessage, OTRL_FRAGMENT_SEND_SKIP,
+                               msg, NULL, &newmessage, OTRL_FRAGMENT_SEND_SKIP,
                                NULL, NULL, NULL);
 #endif
   else {
-    htmlmsg = html_escape(*msg);
     err = otrl_message_sending(userstate, &ops, NULL, ctx->accountname,
 #ifdef HAVE_LIBOTR3
-                               ctx->protocol, ctx->username, htmlmsg, NULL,
+                               ctx->protocol, ctx->username, msg, NULL,
                                &newmessage, NULL, NULL);
 #else
                                // INSTAG XXX
                                ctx->protocol, ctx->username, OTRL_INSTAG_BEST,
-                               htmlmsg, NULL, &newmessage, OTRL_FRAGMENT_SEND_SKIP,
+                               msg, NULL, &newmessage, OTRL_FRAGMENT_SEND_SKIP,
                                NULL, NULL, NULL);
 #endif
-    g_free(htmlmsg);
   }
 
   if (err)
-    *msg = NULL; /*something went wrong, don't send the plain-message! */
+    return NULL; /* something went wrong, don't send the plain-message! */
 
-  if (!err && newmessage) {
-    *msg = g_strdup(newmessage);
-    otrl_message_free(newmessage);
-    if (cb_policy(NULL, ctx) & OTRL_POLICY_REQUIRE_ENCRYPTION ||
-        ctx->msgstate == OTRL_MSGSTATE_ENCRYPTED)
-      return 1;
+  if (cb_policy(NULL, ctx) & OTRL_POLICY_REQUIRE_ENCRYPTION ||
+      ctx->msgstate == OTRL_MSGSTATE_ENCRYPTED)
+    *encryption_status = 1;
+
+  if (!newmessage) {
+    if (*encryption_status == 1)
+      return NULL;  /* This message should have been encrypted */
+
+    /* If not, the encryption was not required - send the original message */
+    return g_strdup(msg);
   }
-  return 0;
+
+  /* Check the new message is not empty */
+  if (newmessage[0]) {
+    rmsg = g_strdup(newmessage);
+  } else {
+    rmsg = NULL;
+    *encryption_status = 0;
+  }
+
+  otrl_message_free(newmessage);
+  return rmsg;
 }
 
 /* Prints OTR connection state */
@@ -855,7 +881,7 @@ static void cb_inject_message(void *opdata, const char *accountname,
                               const char *message)
 {
   if (roster_gettype(recipient) == ROSTER_TYPE_USER)
-    xmpp_send_msg(recipient, message, ROSTER_TYPE_USER, "", TRUE, NULL,
+    xmpp_send_msg(recipient, message, ROSTER_TYPE_USER, NULL, TRUE, NULL,
                   LM_MESSAGE_SUB_TYPE_NOT_SET, NULL);
 }
 
