@@ -121,6 +121,7 @@ static GSList *groups;
 static GSList *unread_list;
 static GHashTable *unread_jids;
 GList *buddylist;
+static gboolean _rebuild_buddylist = FALSE;
 GList *current_buddy;
 GList *alternate_buddy;
 GList *last_activity_buddy;
@@ -437,6 +438,7 @@ GSList *roster_add_user(const char *jid, const char *name, const char *group,
   // #4 Insert node (sorted)
   my_group->list = g_slist_insert_sorted(my_group->list, roster_usr,
                                          (GCompareFunc)&roster_compare_name);
+  buddylist_defer_build();
   return roster_find(jid, jidsearch, type);
 }
 
@@ -472,7 +474,7 @@ void roster_del_user(const char *jid)
 
   // We need to rebuild the list
   if (current_buddy)
-    buddylist_build();
+    buddylist_defer_build();
   // TODO What we could do, too, is to check if the deleted node is
   // current_buddy, in which case we could move current_buddy to the
   // previous (or next) node.
@@ -518,7 +520,7 @@ void roster_free(void)
     groups = NULL;
     // Update (i.e. free) buddylist
     if (buddylist)
-      buddylist_build();
+      buddylist_defer_build();
   }
 }
 
@@ -718,7 +720,7 @@ void roster_msg_setflag(const char *jid, guint special, guint value)
   }
 
   if (buddylist && (new_roster_item || !g_list_find(buddylist, roster_usr)))
-    buddylist_build();
+    buddylist_defer_build();
 
 roster_msg_setflag_return:
   if (unread_list_modified) {
@@ -927,6 +929,7 @@ void buddylist_set_hide_offline_buddies(int hide)
   } else {                      // TRUE  (hide -- andfo)
     display_filter = DFILTER_ONLINE;
   }
+  buddylist_defer_build();
 }
 
 int buddylist_isset_filter(void)
@@ -949,6 +952,11 @@ guchar buddylist_get_filter(void)
   return display_filter;
 }
 
+void buddylist_defer_build(void)
+{
+  _rebuild_buddylist = TRUE;
+}
+
 //  buddylist_build()
 // Creates the buddylist from the roster entries.
 void buddylist_build(void)
@@ -960,6 +968,10 @@ void buddylist_build(void)
   roster *roster_last_activity_buddy = NULL;
   int shrunk_group;
 
+  if (_rebuild_buddylist == FALSE)
+    return;
+  _rebuild_buddylist = FALSE;
+
   // We need to remember which buddy is selected.
   if (current_buddy)
     roster_current_buddy = BUDDATA(current_buddy);
@@ -1094,7 +1106,7 @@ void buddy_setgroup(gpointer rosterdata, char *newgroupname)
   my_newgroup->list = g_slist_insert_sorted(my_newgroup->list, roster_usr,
                                             (GCompareFunc)&roster_compare_name);
 
-  buddylist_build();
+  buddylist_defer_build();
 }
 
 void buddy_setname(gpointer rosterdata, char *newname)
@@ -1118,7 +1130,7 @@ void buddy_setname(gpointer rosterdata, char *newname)
   sl_group = &((roster*)((GSList*)roster_usr->list)->data)->list;
   *sl_group = g_slist_sort(*sl_group, (GCompareFunc)&roster_compare_name);
 
-  buddylist_build();
+  buddylist_defer_build();
 }
 
 const char *buddy_getname(gpointer rosterdata)
@@ -1552,6 +1564,7 @@ GList *buddy_search_jid(const char *jid)
   GList *buddy;
   roster *roster_usr;
 
+  buddylist_build();
   if (!buddylist) return NULL;
 
   for (buddy = buddylist; buddy; buddy = g_list_next(buddy)) {
@@ -1570,6 +1583,7 @@ GList *buddy_search(char *string)
 {
   GList *buddy = current_buddy;
   roster *roster_usr;
+  buddylist_build();
   if (!buddylist || !current_buddy) return NULL;
   for (;;) {
     gchar *jid_locale, *name_locale;
@@ -1749,29 +1763,16 @@ static int unread_jid_del(const char *jid)
   return g_hash_table_remove(unread_jids, jid);
 }
 
-// Helper function for unread_jid_get_list()
-static void add_to_unreadjids(gpointer key, gpointer value, gpointer udata)
-{
-  GList **listp = udata;
-  *listp = g_list_append(*listp, key);
-}
-
 //  unread_jid_get_list()
 // Return the JID list.
 // The content of the list should not be modified or freed.
 // The caller should call g_list_free() after use.
 GList *unread_jid_get_list(void)
 {
-  GList *list = NULL;
-
   if (!unread_jids)
     return NULL;
 
-  // g_hash_table_get_keys() is only in glib >= 2.14
-  //return g_hash_table_get_keys(unread_jids);
-
-  g_hash_table_foreach(unread_jids, add_to_unreadjids, &list);
-  return list;
+  return g_hash_table_get_keys(unread_jids);
 }
 
 /* vim: set et cindent cinoptions=>2\:2(0 ts=2 sw=2:  For Vim users... */
