@@ -556,7 +556,7 @@ LmHandlerResult handle_iq_disco_info(LmMessageHandler *h,
   r = lm_message_new_iq_from_query(m, LM_MESSAGE_SUB_TYPE_RESULT);
   query = lm_message_node_add_child(r->node, "query", NULL);
   lm_message_node_set_attribute(query, "xmlns", NS_DISCO_INFO);
-  tmp = lm_message_node_find_child(m->node, "query");
+  tmp = lm_message_node_get_child(m->node, "query");
   if (tmp) {
     node = lm_message_node_get_attribute(tmp, "node");
     param = node+strlen(MCABBER_CAPS_NODE)+1;
@@ -585,18 +585,18 @@ LmHandlerResult handle_iq_roster(LmMessageHandler *h, LmConnection *c,
   const gchar *from = lm_message_get_from(m);
 
   if (from) {
-    gchar *self_bjid = jidtodisp(lm_connection_get_jid(c));
-    gchar *servername = get_servername(self_bjid, "");
-    if ((!jid_equal(self_bjid, from)) &&
+    const gchar *self_jid = lm_connection_get_jid(c);
+    gchar *servername = get_servername(self_jid, "");
+    if ((!jid_equal(self_jid, from)) &&
        (!servername || strcasecmp(from, servername))) {
       scr_LogPrint(LPRINT_LOGNORM, "Received invalid roster IQ request");
-      g_free(self_bjid);
+      g_free(servername);
       return LM_HANDLER_RESULT_REMOVE_MESSAGE;
     }
-    g_free(self_bjid);
+    g_free(servername);
   }
 
-  y = lm_message_node_find_child(lm_message_node_find_xmlns(m->node, NS_ROSTER),
+  y = lm_message_node_get_child(lm_message_node_find_xmlns(m->node, NS_ROSTER),
                                  "item");
   for ( ; y; y = y->next) {
     char *name_tmp = NULL;
@@ -606,8 +606,8 @@ LmHandlerResult handle_iq_roster(LmMessageHandler *h, LmConnection *c,
     sub = lm_message_node_get_attribute(y, "subscription");
     ask = lm_message_node_get_attribute(y, "ask");
 
-    if (lm_message_node_find_child(y, "group"))
-      group = lm_message_node_get_value(lm_message_node_find_child(y, "group"));
+    if (lm_message_node_get_child(y, "group"))
+      group = lm_message_node_get_value(lm_message_node_get_child(y, "group"));
     else
       group = NULL;
 
@@ -668,8 +668,7 @@ LmHandlerResult handle_iq_roster(LmMessageHandler *h, LmConnection *c,
     lm_message_unref(result);
   }
 
-  buddylist_build();
-  update_roster = TRUE;
+  scr_update_roster();
   if (need_refresh)
     scr_update_buddy_window();
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
